//--------------------------------------------------------------
//  IIR フィルタを利用する直交信号発生器のクラス
//
//  2023/06/15, Copyright (c) 2023 MIKAMI, Naoki
//--------------------------------------------------------------

#include "mbed.h"

#ifndef QUAD_PHASE_OSC_IIR_HPP
#define QUAD_PHASE_OSC_IIR_HPP

class QuadOscIir
{
public:
    // コンストラクタ
    //      f0  周波数
    //      fs  標本化周波数
    //      f0, fs は同じ単位
    QuadOscIir(float f0, float fs)
        : PI2F0TS_(6.283185f*f0/fs), a1_(2.0f*cosf(PI2F0TS_)),
          b1S_(sinf(PI2F0TS_)), b1C_(-cosf(PI2F0TS_)),
          vn1_(1.0f), vn2_(0.0f) {}

    // sin/cos 発生
    void Generate(float &sinX, float &cosX)
    {
        float vn = a1_*vn1_ - vn2_;
        sinX = b1S_*vn1_;
        cosX = vn + b1C_*vn1_;

        vn2_ = vn1_;    // v[n-2] ← v[n-1]
        vn1_ = vn;      // v[n-1] ← v[n]
    }

private:
    const float PI2F0TS_;           // 2πf0×Ts
    const float a1_, b1S_, b1C_;    // 係数
    float vn1_, vn2_;
};
#endif  // QUAD_PHASE_OSC_IIR_HPP
